@@ -27,10 +27,25 @@ class S3Artifact(V1alpha1S3Artifact):
         config = Configuration()
         config.client_side_validation = False
         super().__init__(local_vars_configuration=config, *args, **kwargs)
+        assert isinstance(self.key, str)
+        if not self.key.startswith(s3_config["prefix"]):
+            self.key = s3_config["prefix"] + self.key
         if path_list is None:
             path_list = []
         self.path_list = path_list
 
+    def to_dict(self):
+        d = {"key": self.key}
+        if s3_config["storage_client"] is None:
+            d.update(s3_config)
+        else:
+            d.update(s3_config["storage_client"].to_dict())
+        return d
+
+    @classmethod
+    def from_dict(cls, d):
+        return cls(key=d["key"])
+
     def sub_path(
             self,
             path: str,
@@ -41,6 +56,10 @@ class S3Artifact(V1alpha1S3Artifact):
         artifact.key += path
         return artifact
 
+    def download(self, **kwargs):
+        from .utils import download_artifact
+        download_artifact(self, **kwargs)
+
     def oss(self):
         config = Configuration()
         config.client_side_validation = False
