@@ -20,6 +20,16 @@ config = {
 }
 
 
+def _raise_error(res, op):
+    if res["code"] not in succ_code:
+        if "error" in res:
+            raise RuntimeError("%s failed: %s" % (op, res["error"]["msg"]))
+        elif "message" in res:
+            raise RuntimeError("%s failed: %s" % (op, res["message"]))
+        else:
+            raise RuntimeError("%s failed" % op)
+
+
 def _login(login_url=None, username=None, password=None):
     import requests
     if username is None:
@@ -33,14 +43,7 @@ def _login(login_url=None, username=None, password=None):
     rsp = requests.post(login_url, headers={
                         "Content-type": "application/json"}, json=data)
     res = json.loads(rsp.text)
-    if res["code"] not in succ_code:
-        if "error" in res:
-            raise RuntimeError("Login failed: %s" %
-                               res["error"]["msg"])
-        elif "message" in res:
-            raise RuntimeError("Login failed: %s" % res["message"])
-        else:
-            raise RuntimeError("Login failed")
+    _raise_error(res, "login")
     return res["data"]["token"]
 
 
@@ -92,7 +95,7 @@ class BohriumContext(Context):
     Args:
         username: user name for Bohrium
         password: password for Bohrium
-        login_url: login url for Bohrium
+        bohrium_url: url for Bohrium
         executor: executor
         extra: extra arguments
         authorization: JWT token
@@ -102,13 +105,14 @@ class BohriumContext(Context):
             self,
             username: str = None,
             password: str = None,
-            login_url: str = None,
+            bohrium_url: str = None,
             executor: str = None,
             extra: dict = None,
             authorization: str = None,
     ) -> None:
-        self.login_url = login_url if login_url is not None else \
-            config["bohrium_url"] + "/account/login"
+        self.bohrium_url = bohrium_url if bohrium_url is not None else \
+            config["bohrium_url"]
+        self.login_url = self.bohrium_url + "/account/login"
         self.username = username if username is not None else \
             config["username"]
         self.password = password if password is not None else \
@@ -132,6 +136,8 @@ class BohriumContext(Context):
                 self.extra) if isinstance(self.extra, dict) else self.extra
             template.annotations["workflow.dp.tech/authorization"] = \
                 self.authorization
+            template.annotations["workflow.dp.tech/executor_addr"] = \
+                self.bohrium_url + "/"
             return template
 
         if isinstance(template, (ShellOPTemplate, PythonScriptOPTemplate)):
@@ -189,6 +195,11 @@ class TiefblueClient:
         s3_config["repo_type"] = "oss"
         s3_config["prefix"] = self.prefix
 
+    def to_dict(self):
+        retained_keys = ["bohrium_url",
+                         "tiefblue_url", "username", "project_id"]
+        return {k: self.__dict__[k] for k in retained_keys}
+
     def __getstate__(self):
         retained_keys = ["bohrium_url", "tiefblue_url", "project_id", "token",
                          "prefix"]
@@ -211,6 +222,7 @@ class TiefblueClient:
                 "Authorization": "jwt " + self.authorization},
             params={"projectId": self.project_id})
         res = json.loads(rsp.text)
+        _raise_error(res, "get storage token")
         self.token = res["data"]["token"]
         self.prefix = res["data"]["path"]
 
