@@ -74,6 +74,14 @@ class Slices:
         self.pool_size = pool_size
 
 
+def get_source_code(o):
+    source_lines, start_line = inspect.getsourcelines(o)
+    with open(inspect.getsourcefile(o), "r",
+              encoding="utf-8") as fd:
+        pre_lines = fd.readlines()[:start_line-1]
+    return "".join(pre_lines + source_lines) + "\n"
+
+
 class PythonOPTemplate(PythonScriptOPTemplate):
     """
     Convert from Python class OP to OP template
@@ -370,11 +378,10 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             jsonpickle.dumps(s3_config)
         if op_class.__module__ in ["__main__", "__mp_main__"]:
             try:
-                source_lines, start_line = inspect.getsourcelines(op_class)
-                with open(inspect.getsourcefile(op_class), "r",
-                          encoding="utf-8") as fd:
-                    pre_lines = fd.readlines()[:start_line-1]
-                script += "".join(pre_lines + source_lines) + "\n"
+                if hasattr(op_class, "func"):
+                    script += get_source_code(op_class.func)
+                else:
+                    script += get_source_code(op_class)
             except Exception:
                 import cloudpickle
                 if self.python_packages:
@@ -383,8 +390,13 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                     self.python_packages = set(cloudpickle.__path__)
 
                 script += "import cloudpickle\n"
-                script += "%s = cloudpickle.loads(%s)\n" % \
-                    (class_name, cloudpickle.dumps(op_class))
+                if hasattr(op_class, "func"):
+                    script += "from dflow.python import OP\n"
+                    script += "%s = OP.function(cloudpickle.loads(%s))\n" % \
+                        (class_name, cloudpickle.dumps(op_class.func))
+                else:
+                    script += "%s = cloudpickle.loads(%s)\n" % \
+                        (class_name, cloudpickle.dumps(op_class))
 
         script += "import os, sys, traceback, jsonpickle\n"
         script += "from dflow.python import OPIO, TransientError, FatalError\n"
@@ -392,11 +404,10 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                   " handle_input_parameter\n"
         script += "from dflow.python.utils import handle_output_artifact," \
                   " handle_output_parameter\n"
-        if op_class.__module__ != 'dflow.python.op':
-            script += f"from {op_class.__module__} import {class_name}\n\n"
-        else:  # for class generated by decorator
-            script += op_class.script
-        if op is None:
+        script += f"from {op_class.__module__} import {class_name}\n\n"
+        if hasattr(op_class, "func"):
+            script += "op_obj = %s\n" % class_name
+        elif op is None:
             script += "op_obj = %s()\n" % class_name
         else:
             script += "op_obj = jsonpickle.loads(r'''%s''')\n" % \
