@@ -17,8 +17,7 @@ from .op_template import OPTemplate, PythonScriptOPTemplate, ShellOPTemplate
 from .python import Slices
 from .resource import Resource
 from .util_ops import CheckNumSuccess, CheckSuccessRatio, InitArtifactForSlices
-from .utils import (catalog_of_artifact, merge_dir, randstr, s3_config,
-                    upload_artifact)
+from .utils import catalog_of_artifact, merge_dir, randstr, upload_artifact
 
 try:
     from argo.workflows.client import (V1alpha1Arguments, V1alpha1ContinueOn,
@@ -45,7 +44,9 @@ class FutureLen:
     def __repr__(self):
         return "len(%s)" % self.par
 
-    def get(self):
+    def get(self, context=None):
+        if context is not None and isinstance(self.par, InputParameter):
+            return len(context.inputs.parameters[self.par.name].value)
         return len(self.par.value)
 
 
@@ -57,7 +58,7 @@ class FutureRange:
         args = []
         for i in self.args:
             if isinstance(i, FutureLen):
-                args.append(i.get())
+                args.append(i.get(context))
             elif isinstance(i, (InputParameter, OutputParameter)):
                 args.append(i.value)
             elif isinstance(i, IfExpression):
@@ -72,6 +73,21 @@ class FutureRange:
         return list(range(*args))
 
 
+class ArgoRange(ArgoVar):
+    def __init__(self, end, start=0, step=1):
+        self.end = end
+        self.start = start
+        self.step = step
+        if isinstance(start, (InputParameter, OutputParameter)):
+            start = "sprig.atoi(%s)" % start.expr
+        if isinstance(step, (InputParameter, OutputParameter)):
+            step = "sprig.atoi(%s)" % step.expr
+        if isinstance(end, (InputParameter, OutputParameter)):
+            end = "sprig.atoi(%s)" % end.expr
+        super().__init__("toJson(sprig.untilStep(%s, %s, %s))" %
+                         (start, end, step))
+
+
 def argo_range(
         *args,
 ) -> ArgoVar:
@@ -96,13 +112,28 @@ def argo_range(
         step = args[2]
     else:
         raise TypeError("Expected 1-3 arguments, got %s" % len(args))
-    if isinstance(start, (InputParameter, OutputParameter)):
-        start = "sprig.atoi(%s)" % start.expr
-    if isinstance(step, (InputParameter, OutputParameter)):
-        step = "sprig.atoi(%s)" % step.expr
-    if isinstance(end, (InputParameter, OutputParameter)):
-        end = "sprig.atoi(%s)" % end.expr
-    return ArgoVar("toJson(sprig.untilStep(%s, %s, %s))" % (start, end, step))
+    return ArgoRange(end, start, step)
+
+
+class ArgoSequence:
+    def __init__(self, count, start, end, format):
+        self.count = count
+        self.start = start
+        self.end = end
+        self.format = format
+
+    def convert_to_argo(self):
+        count = self.count
+        start = self.start
+        end = self.end
+        if isinstance(count, ArgoVar):
+            count = "{{=%s}}" % count.expr
+        if isinstance(start, ArgoVar):
+            start = "{{=%s}}" % start.expr
+        if isinstance(end, ArgoVar):
+            end = "{{=%s}}" % end.expr
+        return V1alpha1Sequence(count=count, start=start, end=end,
+                                format=self.format)
 
 
 def argo_sequence(
@@ -123,14 +154,30 @@ def argo_sequence(
             with count, can be an Argo parameter
         format: a printf format string to format the value in the sequence
     """
-    if config["mode"] != "debug":
-        if isinstance(count, ArgoVar):
-            count = "{{=%s}}" % count.expr
-        if isinstance(start, ArgoVar):
-            start = "{{=%s}}" % start.expr
-        if isinstance(end, ArgoVar):
-            end = "{{=%s}}" % end.expr
-    return V1alpha1Sequence(count=count, start=start, end=end, format=format)
+    return ArgoSequence(count=count, start=start, end=end, format=format)
+
+
+class ArgoLen(ArgoVar):
+    def __init__(self, param):
+        self.param = param
+        if isinstance(param, S3Artifact):
+            try:
+                path_list = catalog_of_artifact(param)
+                if path_list:
+                    param.path_list = path_list
+            except Exception:
+                pass
+            super().__init__(str(len(param.path_list)))
+        if isinstance(param, InputArtifact):
+            assert config["save_path_as_parameter"]
+            super().__init__("len(sprig.fromJson(%s))" %
+                             param.get_path_list_parameter())
+        elif isinstance(param, OutputArtifact):
+            assert config["save_path_as_parameter"]
+            super().__init__("len(sprig.fromJson(%s))" %
+                             param.get_path_list_parameter())
+        else:
+            super().__init__("len(sprig.fromJson(%s))" % param.expr)
 
 
 def argo_len(
@@ -144,24 +191,43 @@ def argo_len(
     """
     if config["mode"] == "debug":
         return FutureLen(param)
-    if isinstance(param, S3Artifact):
-        try:
-            path_list = catalog_of_artifact(param)
-            if path_list:
-                param.path_list = path_list
-        except Exception:
-            pass
-        return ArgoVar(str(len(param.path_list)))
-    if isinstance(param, InputArtifact):
-        assert config["save_path_as_parameter"]
-        return ArgoVar("len(sprig.fromJson(%s))" %
-                       param.get_path_list_parameter())
-    elif isinstance(param, OutputArtifact):
-        assert config["save_path_as_parameter"]
-        return ArgoVar("len(sprig.fromJson(%s))" %
-                       param.get_path_list_parameter())
-    else:
-        return ArgoVar("len(sprig.fromJson(%s))" % param.expr)
+    return ArgoLen(param)
+
+
+class ArgoSum:
+    def __init__(self, param):
+        self.param = param
+        self.expr = "sum(%s)" % param
+
+
+def argo_sum(
+        param: ArgoVar,
+) -> ArgoSum:
+    """
+    Return the sum of a list of integers which is an Argo parameter
+
+    Args:
+        param: the Argo parameter which is a list of integers
+    """
+    return ArgoSum(param)
+
+
+class ArgoConcat:
+    def __init__(self, param):
+        self.param = param
+        self.expr = "concat(%s)" % param
+
+
+def argo_concat(
+        param: ArgoVar,
+) -> ArgoConcat:
+    """
+    Return the concatenation of a list of lists which is an Argo parameter
+
+    Args:
+        param: the Argo parameter which is a list of lists
+    """
+    return ArgoConcat(param)
 
 
 class Step:
@@ -214,6 +280,7 @@ class Step:
             util_command: Union[str, List[str]] = None,
             parallelism: int = None,
             slices: Slices = None,
+            register_output_artifacts: Dict[str, dict] = None,
             **kwargs,
     ) -> None:
         self.name = name
@@ -251,6 +318,7 @@ class Step:
             util_command = [util_command]
         self.util_command = util_command
         self.parallelism = parallelism
+        self.register_output_artifacts = register_output_artifacts
 
         if hasattr(self.template, "python_packages") and \
                 self.template.python_packages:
@@ -377,19 +445,44 @@ class Step:
 
             self.template = new_template
 
-        if hasattr(self.template, "slices") and self.template.slices is not \
-                None and (self.template.slices.output_artifact or (
-                    self.template.slices.sub_path and
-                    self.template.slices.input_artifact)):
+        sum_var = None
+        if isinstance(self.with_param, ArgoRange) and \
+                isinstance(self.with_param.end, ArgoSum):
+            sum_var = self.with_param.end.param
+
+        if self.with_sequence is not None and \
+                isinstance(self.with_sequence.count, ArgoSum):
+            sum_var = self.with_sequence.count.param
+
+        concat_var = None
+        if isinstance(self.with_param, ArgoRange) and \
+                isinstance(self.with_param.end, ArgoLen) and \
+                isinstance(self.with_param.end.param, ArgoConcat):
+            concat_var = self.with_param.end.param.param
+
+        if self.with_sequence is not None and \
+                isinstance(self.with_sequence.count, ArgoLen) and \
+                isinstance(self.with_sequence.count.param, ArgoConcat):
+            concat_var = self.with_sequence.count.param.param
+
+        sliced_output_artifact = self.template.slices.output_artifact if \
+            hasattr(self.template, "slices") and \
+            self.template.slices is not None else []
+
+        sliced_input_artifact = self.template.slices.input_artifact if \
+            hasattr(self.template, "slices") and \
+            self.template.slices is not None and \
+            self.template.slices.sub_path else []
+
+        if sliced_output_artifact or sliced_input_artifact or \
+                sum_var is not None or concat_var is not None:
             if new_template is None:
                 new_template = deepcopy(self.template)
                 new_template.name = self.template.name + "-" + randstr()
             init_template = InitArtifactForSlices(
                 new_template.name, self.util_image, self.util_command,
-                self.util_image_pull_policy, self.key,
-                self.template.slices.output_artifact,
-                self.template.slices.sub_path,
-                self.template.slices.input_artifact)
+                self.util_image_pull_policy, self.key, sliced_output_artifact,
+                sliced_input_artifact, sum_var, concat_var)
             if self.key is not None:
                 new_template.inputs.parameters["dflow_group_key"] = \
                     InputParameter(value="")
@@ -397,11 +490,10 @@ class Step:
                     value=re.sub("{{item.*}}", "group", str(self.key)))
                 # For the case of reusing sliced steps, ensure that the output
                 # artifacts are reused
-                for name in new_template.slices.output_artifact:
+                for name in sliced_output_artifact:
                     new_template.outputs.artifacts[name].save.append(
-                        S3Artifact(key="%s{{workflow.name}}/{{inputs."
-                                   "parameters.dflow_group_key}}/%s" % (
-                                       s3_config["prefix"], name)))
+                        S3Artifact(key="{{workflow.name}}/{{inputs."
+                                   "parameters.dflow_group_key}}/%s" % name))
 
                     def merge_output_artifact(art):
                         step = art.step
@@ -413,9 +505,8 @@ class Step:
                                 value="{{inputs.parameters.dflow_group_key}}")
                         template.outputs.artifacts[art.name].save.append(
                             S3Artifact(
-                                key="%s{{workflow.name}}/{{inputs."
-                                "parameters.dflow_group_key}}/%s" % (
-                                    s3_config["prefix"], name)))
+                                key="{{workflow.name}}/{{inputs."
+                                "parameters.dflow_group_key}}/%s" % name))
 
                     if new_template.outputs.artifacts[name]._from is not \
                             None:
@@ -430,7 +521,7 @@ class Step:
             else:
                 new_template.inputs.parameters["dflow_artifact_key"] = \
                     InputParameter(value="")
-                for name in new_template.slices.output_artifact:
+                for name in sliced_output_artifact:
                     new_template.outputs.artifacts[name].save.append(
                         S3Artifact(key="{{inputs.parameters."
                                    "dflow_artifact_key}}/%s" % name))
@@ -444,9 +535,8 @@ class Step:
                             InputParameter(value="{{inputs.parameters."
                                            "dflow_artifact_key}}")
                         template.outputs.artifacts[art.name].save.append(
-                            S3Artifact(
-                                key="{{inputs.parameters."
-                                "dflow_artifact_key}}/%s" % name))
+                            S3Artifact(key="{{inputs.parameters."
+                                       "dflow_artifact_key}}/%s" % name))
 
                     if new_template.outputs.artifacts[name]._from is not \
                             None:
@@ -474,9 +564,8 @@ class Step:
                     value=self.prepare_step.outputs.parameters[
                         "dflow_artifact_key"])
 
-            if new_template.slices.sub_path and \
-                    new_template.slices.input_artifact:
-                for name in new_template.slices.input_artifact:
+            if sliced_input_artifact:
+                for name in sliced_input_artifact:
                     self.inputs.parameters["dflow_%s_sub_path" %
                                            name].value = "{{item.%s}}" % name
                     # step cannot resolve
@@ -496,10 +585,54 @@ class Step:
                 self.with_param = self.prepare_step.outputs.parameters[
                     "dflow_slices_path"]
 
-            for name in new_template.slices.output_artifact:
+            for name in sliced_output_artifact:
                 self.outputs.artifacts[name].redirect = \
                     self.prepare_step.outputs.artifacts[name]
 
+            if isinstance(self.with_param, ArgoRange) and \
+                    isinstance(self.with_param.end, ArgoSum):
+                name = sum_var.name
+                self.prepare_step.inputs.parameters[name] = InputParameter(
+                    value=str(sum_var))
+                self.with_param = ArgoRange(
+                    self.prepare_step.outputs.parameters["sum_%s" % name],
+                    self.with_param.start,
+                    self.with_param.step)
+
+            if self.with_sequence is not None and \
+                    isinstance(self.with_sequence.count, ArgoSum):
+                name = sum_var.name
+                self.prepare_step.inputs.parameters[name] = InputParameter(
+                    value=str(sum_var))
+                self.with_sequence = argo_sequence(
+                    self.prepare_step.outputs.parameters["sum_%s" % name],
+                    self.with_sequence.start, self.with_sequence.end,
+                    self.with_sequence.format)
+
+            if isinstance(self.with_param, ArgoRange) and \
+                    isinstance(self.with_param.end, ArgoLen) and \
+                    isinstance(self.with_param.end.param, ArgoConcat):
+                name = concat_var.name
+                self.prepare_step.inputs.parameters[name] = InputParameter(
+                    value=str(concat_var))
+                self.with_param = ArgoRange(
+                    argo_len(self.prepare_step.outputs.parameters[
+                        "concat_%s" % name]),
+                    self.with_param.start,
+                    self.with_param.step)
+
+            if self.with_sequence is not None and \
+                    isinstance(self.with_sequence.count, ArgoLen) and \
+                    isinstance(self.with_sequence.count.param, ArgoConcat):
+                name = concat_var.name
+                self.prepare_step.inputs.parameters[name] = InputParameter(
+                    value=str(concat_var))
+                self.with_sequence = argo_sequence(
+                    argo_len(self.prepare_step.outputs.parameters[
+                        "concat_%s" % name]),
+                    self.with_sequence.start, self.with_sequence.end,
+                    self.with_sequence.format)
+
         pvc_arts = []
         for art in self.inputs.artifacts.values():
             if isinstance(art.source, PVC):
@@ -744,22 +877,22 @@ class Step:
                 if self.with_sequence.start is not None:
                     steps.inputs.parameters["dflow_sequence_start"] = \
                         InputParameter()
-                    step.with_sequence.start = "{{=%s}}" % \
-                        steps.inputs.parameters["dflow_sequence_start"].expr
+                    step.with_sequence.start = steps.inputs.parameters[
+                        "dflow_sequence_start"]
                     self.inputs.parameters["dflow_sequence_start"] = \
                         InputParameter(value=self.with_sequence.start)
                 if self.with_sequence.end is not None:
                     steps.inputs.parameters["dflow_sequence_end"] = \
                         InputParameter()
-                    step.with_sequence.end = "{{=%s}}" % \
-                        steps.inputs.parameters["dflow_sequence_end"].expr
+                    step.with_sequence.end = steps.inputs.parameters[
+                        "dflow_sequence_end"]
                     self.inputs.parameters["dflow_sequence_end"] = \
                         InputParameter(value=self.with_sequence.end)
                 if self.with_sequence.count is not None:
                     steps.inputs.parameters["dflow_sequence_count"] = \
                         InputParameter()
-                    step.with_sequence.count = "{{=%s}}" % \
-                        steps.inputs.parameters["dflow_sequence_count"].expr
+                    step.with_sequence.count = steps.inputs.parameters[
+                        "dflow_sequence_count"]
                     self.inputs.parameters["dflow_sequence_count"] = \
                         InputParameter(value=self.with_sequence.count)
                 self.with_sequence = None
@@ -868,6 +1001,20 @@ class Step:
                                                             str):
             self.with_param = jsonpickle.dumps(list(self.with_param))
 
+        if self.register_output_artifacts is not None:
+            assert hasattr(self.template, "op"), "Only Python OP is supported"\
+                " for registering output artifacts"
+            new_template = deepcopy(self.template)
+            new_template.script += "from registry import Dataset\n"
+            new_template.script += "from dflow import S3Artifact\n"
+            for k, v in self.register_output_artifacts.items():
+                new_template.script += """ds = Dataset(
+    namespace='{{workflow.name}}/{{pod.name}}',
+    name='%s',
+    location=S3Artifact(key=op_obj.get_output_artifact_storage_key('%s')),
+    **%s)\n""" % (k, k, v)
+                new_template.script += "ds.insert()\n"
+
         if context is not None:
             self.template = context.render(self.template)
 
@@ -892,7 +1039,8 @@ class Step:
                 parameters=self.argo_parameters,
                 artifacts=self.argo_artifacts
             ), when=self.when, with_param=self.with_param,
-            with_sequence=self.with_sequence,
+            with_sequence=None if self.with_sequence is None else
+            self.with_sequence.convert_to_argo(),
             continue_on=V1alpha1ContinueOn(failed=self.continue_on_failed)
         )
 
@@ -919,7 +1067,7 @@ class Step:
         for name, par in parameters.items():
             value = par.value
             if isinstance(value, FutureLen):
-                par.value = value.get()
+                par.value = value.get(context)
             elif isinstance(value, (InputParameter, OutputParameter)):
                 par.value = get_var(value, context).value
             elif isinstance(value, str):
@@ -1032,20 +1180,20 @@ class Step:
                 if self.with_sequence.start is not None:
                     start = self.with_sequence.start
                     if isinstance(start, FutureLen):
-                        start = start.get()
+                        start = start.get(context)
                     if isinstance(start, (InputParameter, OutputParameter)):
                         start = start.value
                 if self.with_sequence.count is not None:
                     count = self.with_sequence.count
                     if isinstance(count, FutureLen):
-                        count = count.get()
+                        count = count.get(context)
                     if isinstance(count, (InputParameter, OutputParameter)):
                         count = count.value
                     sequence = list(range(start, start + count))
                 if self.with_sequence.end is not None:
                     end = self.with_sequence.end
                     if isinstance(end, FutureLen):
-                        end = end.get()
+                        end = end.get(context)
                     if isinstance(end, (InputParameter, OutputParameter)):
                         end = end.value
                     if end >= start:
@@ -1093,7 +1241,7 @@ class Step:
                     if isinstance(value, str):
                         par.value.append(value)
                     else:
-                        par.value.append(jsonpickle.loads(value))
+                        par.value.append(jsonpickle.dumps(value))
             for name, art in self.outputs.artifacts.items():
                 for save in self.template.outputs.artifacts[name].save:
                     if isinstance(save, S3Artifact):
