@@ -34,6 +34,16 @@ except Exception:
     pass
 
 
+class DockerSecret:
+    def __init__(self, registry, username, password, name=None):
+        self.registry = registry
+        self.username = username
+        self.password = password
+        if name is None:
+            name = "dflow-%s" % randstr()
+        self.name = name
+
+
 class Workflow:
     """
     Workflow
@@ -85,7 +95,8 @@ class Workflow:
             annotations: Dict[str, str] = None,
             parallelism: int = None,
             pod_gc_strategy: str = None,
-            image_pull_secrets: Union[str, List[str]] = None,
+            image_pull_secrets: Union[str, DockerSecret,
+                                      List[Union[str, DockerSecret]]] = None,
             artifact_repo_key: str = None,
     ) -> None:
         self.host = host if host is not None else config["host"]
@@ -100,7 +111,8 @@ class Workflow:
         self.annotations = annotations
         self.parallelism = parallelism
         self.pod_gc_strategy = pod_gc_strategy
-        if isinstance(image_pull_secrets, str):
+        if image_pull_secrets is not None and not isinstance(
+                image_pull_secrets, list):
             image_pull_secrets = [image_pull_secrets]
         self.image_pull_secrets = image_pull_secrets
         self.artifact_repo_key = artifact_repo_key if artifact_repo_key is \
@@ -137,25 +149,9 @@ class Workflow:
         self.argo_templates = {}
         self.pvcs = {}
 
-        if self.k8s_api_server is not None:
-            k8s_configuration = kubernetes.client.Configuration(
-                host=self.k8s_api_server)
-            k8s_configuration.verify_ssl = False
-            if self.token is None:
-                k8s_client = kubernetes.client.ApiClient(
-                    k8s_configuration)
-            else:
-                k8s_client = kubernetes.client.ApiClient(
-                    k8s_configuration, header_name='Authorization',
-                    header_value='Bearer %s' % self.token)
-            self.core_v1_api = kubernetes.client.CoreV1Api(k8s_client)
-        else:
-            kubernetes.config.load_kube_config(
-                config_file=self.k8s_config_file)
-            self.core_v1_api = kubernetes.client.CoreV1Api()
-
         if self.artifact_repo_key is not None:
-            cm = self.core_v1_api.read_namespaced_config_map(
+            core_v1_api = self.get_k8s_core_v1_api()
+            cm = core_v1_api.read_namespaced_config_map(
                 namespace=self.namespace, name="artifact-repositories")
             repo = yaml.full_load(cm.data[self.artifact_repo_key])
             s3_config["repo"] = repo
@@ -170,6 +166,24 @@ class Workflow:
                 if s3["keyFormat"].endswith(t):
                     s3_config["repo_prefix"] = s3["keyFormat"][:-len(t)]
 
+    def get_k8s_core_v1_api(self):
+        if self.k8s_api_server is not None:
+            k8s_configuration = kubernetes.client.Configuration(
+                host=self.k8s_api_server)
+            k8s_configuration.verify_ssl = False
+            if self.token is None:
+                k8s_client = kubernetes.client.ApiClient(
+                    k8s_configuration)
+            else:
+                k8s_client = kubernetes.client.ApiClient(
+                    k8s_configuration, header_name='Authorization',
+                    header_value='Bearer %s' % self.token)
+            return kubernetes.client.CoreV1Api(k8s_client)
+        else:
+            kubernetes.config.load_kube_config(
+                config_file=self.k8s_config_file)
+            return kubernetes.client.CoreV1Api()
+
     def __enter__(self) -> 'Workflow':
         GLOBAL_CONTEXT.in_context = True
         GLOBAL_CONTEXT.current_workflow = self
@@ -362,7 +376,8 @@ class Workflow:
                 config_map = kubernetes.client.V1ConfigMap(
                     data=data, metadata=kubernetes.client.V1ObjectMeta(
                         name="dflow-%s-%s" % (self.id, step.key)))
-                self.core_v1_api.create_namespaced_config_map(
+                core_v1_api = self.get_k8s_core_v1_api()
+                core_v1_api.create_namespaced_config_map(
                     namespace=self.namespace, body=config_map)
             self.handle_template(
                 self.entrypoint, memoize_prefix=self.id,
@@ -391,6 +406,21 @@ class Workflow:
             metadata = V1ObjectMeta(
                 generate_name=self.name + '-', annotations=self.annotations)
 
+        if self.image_pull_secrets is not None:
+            for i, s in enumerate(self.image_pull_secrets):
+                if isinstance(s, DockerSecret):
+                    data = {".dockerconfigjson": json.dumps({
+                        "auths": {s.registry: {
+                            "username": s.username, "password": s.password}}})}
+                    secret = kubernetes.client.V1Secret(
+                        string_data=data,
+                        metadata=kubernetes.client.V1ObjectMeta(name=s.name),
+                        type="kubernetes.io/dockerconfigjson")
+                    core_v1_api = self.get_k8s_core_v1_api()
+                    core_v1_api.create_namespaced_secret(
+                        namespace=self.namespace, body=secret)
+                    self.image_pull_secrets[i] = s.name
+
         return V1alpha1Workflow(
             metadata=metadata,
             spec=V1alpha1WorkflowSpec(
